#include <SDL/SDL.h>
#include <SDL/SDL_image.h>

#include "bdancer.h"

#define COLORKEY(s) SDL_MapRGB((s)->format,255,0,255)
#define SET_COLOR_KEY(s) SDL_SetColorKey(s,SDL_SRCCOLORKEY,COLORKEY(s))

extern const int offsetx[SEQ_LEN],offsety[SEQ_LEN];

SDL_Surface *legs[SEQ_LEN],*face[SEQ_LEN],*arml[SEQ_LEN],
  *armr[SEQ_LEN],*feet[SEQ_LEN],*screen,*bgnd;

static int bgoffx=0,bgoffy=0;
static int armoffy=0;
static int faceoffy=0;
static int armw=0;

void rect_center(const SDL_Surface *screen, SDL_Rect *r, int w, int h);
int blit_center(SDL_Surface *screen, SDL_Surface *src, int xoff, int yoff);
int load_nana(SDL_Surface *surf[],const char *cmdstr, int min, int max);
void free_nana(SDL_Surface *surf[], int len);

void blit_nana(SDL_Surface *out, unsigned int m)
{
  m%=SEQ_LEN;

  SDL_FillRect(out,NULL,SDL_MapRGB(out->format,255,0,255));
  blit_center(out,legs[m],0,0);
  blit_center(out,arml[m],armw,armoffy);
  blit_center(out,armr[m],-armw,armoffy);
  blit_center(out,bgnd,offsetx[m]+bgoffx,offsety[m]+bgoffy);
  blit_center(out,face[m],0,faceoffy);
  blit_center(out,feet[m],0,0);
}

int main(int argc, char *argv[])
{
  SDL_Event event;
  int running=1;


  if(argc != 2)
  {
    fprintf(stderr,"Syntax:  bdancer bgnd.bmp\n");
    return(4);
  }

  if(SDL_Init(SDL_INIT_VIDEO)<0)
    return(-1);

  screen=SDL_SetVideoMode(128,128,32,SDL_ANYFORMAT);
  if(screen==NULL)
  { SDL_Quit(); return(1); };

  SDL_WM_SetCaption("Bdancer","Bdancer");

  bgnd=IMG_Load(argv[1]);

  if(bgnd==NULL)
  { SDL_Quit(); return(5); }

  {
    SDL_Surface *tmp=SDL_DisplayFormat(bgnd);
    SDL_FreeSurface(bgnd);
    bgnd=tmp;
    SET_COLOR_KEY(bgnd);
  }

  if(load_nana(legs,LEGS_SEQ,SEQ_MIN,SEQ_MAX)<0)
  {
    fprintf(stderr,"Couldn't load legs\n");
    SDL_Quit();
    return(2);
  }

  if(load_nana(face,FACE_SEQ,SEQ_MIN,SEQ_MAX)<0)
  {
    fprintf(stderr,"Couldn't load legs\n");
    free_nana(legs,SEQ_LEN);
    SDL_Quit();
    return(3);
  }

  if(load_nana(arml,ARML_SEQ,SEQ_MIN,SEQ_MAX)<0)
  {
    fprintf(stderr,"Couldn't load armls\n");
    free_nana(legs,SEQ_LEN);
    free_nana(face,SEQ_LEN);
    SDL_Quit();
    return(3);
  }

  if(load_nana(armr,ARMR_SEQ,SEQ_MIN,SEQ_MAX)<0)
  {
    fprintf(stderr,"Couldn't load armls\n");
    free_nana(legs,SEQ_LEN);
    free_nana(face,SEQ_LEN);
    free_nana(arml,SEQ_LEN);
    SDL_Quit();
    return(3);
  }

  if(load_nana(feet,FEET_SEQ,SEQ_MIN,SEQ_MAX)<0)
  {
    fprintf(stderr,"Couldn't load armls\n");
    free_nana(legs,SEQ_LEN);
    free_nana(face,SEQ_LEN);
    free_nana(arml,SEQ_LEN);
    free_nana(armr,SEQ_LEN);
    SDL_Quit();
    return(4);
  }

  fprintf(stderr,"Loaded nanananananana!\n");

  while(running)
  {
    static int seq=0,ticks=-1;

    while(SDL_PollEvent(&event))
      switch(event.type)
      {
      case SDL_KEYDOWN:
        switch(event.key.keysym.sym)
        {
        case SDLK_w:
          if(SDL_GetModState()&(KMOD_LSHIFT|KMOD_RSHIFT))
            armw++;
          else
            armw--;
          break;
        case SDLK_PRINT:
          {
            int m;
            SDL_Surface *out=SDL_DisplayFormat(SDL_GetVideoSurface());
            for(m=0; m<SEQ_LEN; m++)
            {
              char buf[64];
              blit_nana(out,m);
              sprintf(buf,"out%02d.bmp",SEQ_LEN - (m+1));
              fprintf(stderr,"Saving %s\n",buf);
              SDL_SaveBMP(out,buf);
            }
            SDL_FreeSurface(out);
          }
          break;

        case SDLK_KP9:
          armoffy--;
          break;
        case SDLK_KP3:
          armoffy++;
          break;
        case SDLK_KP7:
          faceoffy--;
          break;
        case SDLK_KP1:
          faceoffy++;
          break;
        case SDLK_KP4:
          bgoffx--;
          break;
        case SDLK_KP6:
          bgoffx++;
          break;
        case SDLK_KP8:
          bgoffy--;
          break;
        case SDLK_KP2:
          bgoffy++;
          break;
        default:
          break;
        }
        break;
      case SDL_QUIT:  running=0;
      default:        break;
      }

    if(ticks<0)
      ticks=SDL_GetTicks();
    else if((SDL_GetTicks()-ticks) > 100)
    {
      seq++;
      ticks+=100;

      blit_nana(screen,seq);
      SDL_Flip(screen);
    }
    else
      SDL_Delay(10);
  }

  free_nana(legs,SEQ_LEN);
  free_nana(face,SEQ_LEN);
  free_nana(arml,SEQ_LEN);
  free_nana(armr,SEQ_LEN);
  SDL_Quit();
  return(0);
}

int blit_center(SDL_Surface *screen, SDL_Surface *src, int xoff, int yoff)
{
  SDL_Rect r;
  rect_center(screen,&r,src->w,src->h);
  r.x+=xoff;
  r.y+=yoff;
  return(SDL_BlitSurface(src,NULL,screen,&r));
}

void rect_center(const SDL_Surface *screen, SDL_Rect *r, int w, int h)
{
  r->x = (screen->w/2) - (w/2);
  r->y = (screen->h/2) - (h/2);
  r->w = w;
  r->h = h;
}

void free_nana(SDL_Surface *surf[], int len)
{
  len--;
  while(len>=0)
  {
    SDL_FreeSurface(surf[len]);
    len--;
  }
}

int load_nana(SDL_Surface *surf[],const char *cmdstr, int min, int max)
{
  int n;
  char buf[64];
  for(n=min;  n<=max; n++)
  {
    SDL_Surface *tmp;
    sprintf(buf,cmdstr,n);
    tmp=IMG_Load(buf);
    if(tmp==NULL)
    {
      int m;
      for(m=n-1; m>=min; m--) SDL_FreeSurface(surf[n]);
      return(-1);      
    }

    surf[n]=SDL_DisplayFormat(tmp);
    SDL_FreeSurface(tmp);
    SET_COLOR_KEY(surf[n]);
  }

  return(0);
}

const int offsetx[SEQ_LEN]={ 3, 1, 0, 0,-1,-3, 0, 0};
const int offsety[SEQ_LEN]={ 5, 3, 0, 2, 5, 3, 0, 2};
